/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.softfloat;

import ru.ispras.softfloat.JSoftFloatUtils.FloatDetectTininess;
import ru.ispras.softfloat.JSoftFloatUtils.FloatExceptionFlags;
import ru.ispras.softfloat.JSoftFloatUtils.FloatRoundingMode;

/**
 * The half-precision floating-point operations.
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */

public class Float16Functions {
  /**
   * Returns the fraction bits of the half-precision floating-point value 'a'.
   * 
   * @param a the half-precision floating-point value
   * 
   * @return the fraction bits
   */
  public static int extractFloat16Frac(final Float16 a) {
    return a.float16b & 0x03FF;
  }

  /**
   * Returns the exponent bits of the half-precision floating-point value `a'.
   * 
   * @param a the half-precision floating-point value
   * 
   * @return the exponent bits
   */
  public static int extractFloat16Exp(final Float16 a) {
    return a.float16b >>> 10 & 0x1F;
  }

  /**
   * Returns the sign bit of the half-precision floating-point value `a'.
   * 
   * @param a the half-precision floating-point value
   * 
   * @return the sign bit
   */
  public static boolean extractFloat16Sign(final Float16 a) {
    return a.float16b >>> 15 != 0;
  }

  public static boolean softfloat_isSigNaNF16UI(int uiA) {
	  return ((((uiA) & 0x7E00) == 0x7C00) & (((uiA) & 0x01FF) != 0));  
  }

  // float64_t f16_to_f64( float16_t a )
  public static double f16_to_f64(Float16 a) {
    boolean aSign; // flag
    int aExp; // int16
    int aFrac; // bits32

    aExp = extractFloat16Exp(a);
    aFrac = extractFloat16Frac(a);
    aSign = extractFloat16Sign(a);

    if (aExp == 0x1F) {
      double float64;
      if (aFrac != 0) {
        CommonNaNT commonNaNT = softfloat_f16UIToCommonNaN(a.float16b);
        float64 = JSoftFloat.commonNaNToFloat64(commonNaNT);
      } else {
        float64 = JSoftFloat.packFloat64(aSign, (short) 0x7FF, 0);
      }

      return float64;
    }
    if (aExp == 0) {
      if (aFrac == 0) {
        double float64 = JSoftFloat.packFloat64(aSign, (short) 0, 0);
        return float64;
      }
      aExp = 0xFF;
      // frac = aFrac;
    }

    double float64 = JSoftFloat.packFloat64(aSign, (short) (aExp + 0x3F0), ((long) aFrac) << 42);
    return float64;
  }

  public static Float16 f64_to_f16(double a) {
    boolean aSign;
    long aFrac, frac16;
    long aExp;

    aFrac = JSoftFloat.extractFloat64Frac(a);
    aExp = JSoftFloat.extractFloat64Exp(a);
    aSign = JSoftFloat.extractFloat64Sign(a);

    Float16 output;

    if (aExp == 0x7FF) {
      if (aFrac != 0) {
        CommonNaNT commonNaNT = JSoftFloat.float64ToCommonNaN(a);
        output = softfloat_f16UIToCommonNaN(commonNaNT);
      } else {
        output = packToF16UI(aSign, (short) 0x1F, (short) 0);
      }
      return output;
    }

    frac16 = softfloat_shortShiftRightJam64(aFrac, (short) 38);
    if (!(aExp != 0 | frac16 != 0)) {
      output = packToF16UI(aSign, (short) 0, (short) 0);
      return output;
    }

    // return softfloat_roundPackToF16( sign, exp - 0x3F1, frac16 | 0x4000 );
    return softfloat_roundPackToF16(aSign, (short) (aExp - 0x3F1), (short) (frac16 | 0x4000));
  }

  public static Float128 f16_to_f128(Float16 a) {
    boolean aSign; // flag
    int aExp; // int16
    int aFrac; // bits32

    aExp = extractFloat16Exp(a);
    aFrac = extractFloat16Frac(a);
    aSign = extractFloat16Sign(a);

    Float128 float128 = new Float128();
    if (aExp == 0x1F) {
      if (aFrac != 0) {
        CommonNaNT commonNaNT = softfloat_f16UIToCommonNaN(a.float16b);
        float128 = JSoftFloat.commonNaNToFloat128(commonNaNT);
      } else {
        float128.high = packToF128UI64(aSign, (long) 0x7FFF, (long) 0);
        float128.low = 0;
      }

      return float128;
    }
    if (aExp == 0) {
      if (aFrac == 0) {
        float128.high = packToF128UI64(aSign, (long) 0x7FFF, (long) 0);
        float128.low = 0;
        return float128;
      }
      aExp = 0xFF;
    }

    float128.high = packToF128UI64(aSign, (long) (aExp + 0x3FF0), ((long) aFrac) << 38);
    float128.low = 0;
    return float128;
  }

  public static float f16_to_f32(Float16 a) {
    boolean aSign; // flag
    int aExp; // int16
    int aFrac; // bits32

    aExp = extractFloat16Exp(a);
    aFrac = extractFloat16Frac(a);
    aSign = extractFloat16Sign(a);

    if (aExp == 0x1F) {
      float float32;
      if (aFrac != 0) {
        CommonNaNT commonNaNT = softfloat_f16UIToCommonNaN(a.float16b);
        float32 = JSoftFloat.commonNaNToFloat32(commonNaNT);
      } else {
        float32 = JSoftFloat.packFloat32(aSign, (short) 0xFF, 0);
      }

      return float32;
    }
    if (aExp == 0) {
      if (aFrac == 0) {
        float float32 = JSoftFloat.packFloat32(aSign, (short) 0, 0);
        return float32;
      }
      aExp = 0xFF;
      // frac = aFrac;
    }

    float float32 = JSoftFloat.packFloat32(aSign, (short) (aExp + 0x70), aFrac << 13);
    return float32;
  }

  public static Float16 f32_to_f16(float a) {
    boolean aSign;
    int aFrac, frac16;
    short aExp;

    aFrac = JSoftFloat.extractFloat32Frac(a);
    aExp = JSoftFloat.extractFloat32Exp(a);
    aSign = JSoftFloat.extractFloat32Sign(a);

    Float16 output;

    if (aExp == 0xFF) {
      if (aFrac != 0) {
        CommonNaNT commonNaNT = JSoftFloat.float32ToCommonNaN(a);
        output = softfloat_f16UIToCommonNaN(commonNaNT);
      } else {
        output = packToF16UI(aSign, (short) 0x1F, (short) 0);
      }
      return output;
    }

    frac16 = aFrac >>> 9 | ((aFrac & 0x1FF) != 0 ? 1 : 0);
    if (!(aExp != 0 | frac16 != 0)) {
      output = packToF16UI(aSign, (short) 0, (short) 0);
      return output;
    }

    // return softfloat_roundPackToF16( sign, exp - 0x3F1, frac16 | 0x4000 );
    return softfloat_roundPackToF16(aSign, (short) (aExp - 0x71), (short) (frac16 | 0x4000));
  }

  public static Long packToF128UI64(boolean sign, long exp, long sig64) {
    return ((sign ? (long) 1 : (long) 0) << 63) + ((exp) << 48) + sig64;
  }

  public static long softfloat_shortShiftRightJam64(long a, final short dist) {
    // return a>>dist | ((a & (((uint_fast64_t) 1<<dist) - 1)) != 0);
    return a >>> dist | ((a & (((long) 1 << dist) - 1)) != 0 ? 1 : 0);
  }

  public static Float16 packToF16UI(boolean sign, short exp, short sig) {
    // (((uint16_t) (sign)<<15) + ((uint16_t) (exp)<<10) + (sig))
    return new Float16(
        0xffff & (((sign ? (short) 1 : (short) 0) << 15) + ((short) (exp) << 10) + (short) (sig)));
  }

  public static CommonNaNT softfloat_f16UIToCommonNaN(int uiA) {
    CommonNaNT zPtr = new CommonNaNT();

    if (softfloat_isSigNaNF16UI(uiA)) {
      JSoftFloat.float_raise(FloatExceptionFlags.float_flag_invalid.getValue());
    }

    zPtr.sign = extractFloat16Sign(new Float16(uiA));
    zPtr.high = Long.valueOf(uiA) << 54;
    zPtr.low = 0;

    return zPtr;
  }

  public static Float16 softfloat_f16UIToCommonNaN(CommonNaNT commonNaNT) {
    // return (uint_fast16_t) aPtr->sign<<15 | 0x7E00 | aPtr->v64>>54;
    return new Float16(
        0xffff & (((commonNaNT.sign ? (short) 1 : (short) 0) << 15) | 0x7E00 | (short) (commonNaNT.high >> 54)));
  }

  /**
   * Packs the sign `sign', the exponent `exp', and the significand `sig' into a half-precision
   * floating-point value, returning the result.
   * 
   * @param sign
   * @param exp
   * @param sig
   * 
   * @return Float16
   */
  public static Float16 softfloat_roundPackToF16(boolean sign, short exp, short sig) {
    FloatRoundingMode roundingMode; // int8
    boolean roundNearestEven; // flag
    short roundIncrement; // int16
    short roundBits; // int16
    boolean isTiny; // flag

    roundingMode = JSoftFloat.float_rounding_mode;
    roundNearestEven = (roundingMode == FloatRoundingMode.float_round_nearest_even);
    roundIncrement = 0x8;
    if (!roundNearestEven && (roundingMode == FloatRoundingMode.softfloat_round_near_maxMag)) {
      roundIncrement =
          (roundingMode == (sign ? FloatRoundingMode.float_round_down
              : FloatRoundingMode.float_round_up)) ? (short) 0xF : (short) 0;
    }

    roundBits = (short) (sig & 0xF);

    if (0x1D <= (exp & 0xffff)) {
      if (exp < 0) {
        isTiny =
            (JSoftFloat.float_detect_tininess == FloatDetectTininess.float_tininess_before_rounding)
                || (exp < -1) || (sig + roundIncrement < 0x8000);
        sig = (short) JSoftFloatUtils.shift32RightJamming((int) sig, (short) (-exp));// , zSig);
        exp = 0;
        roundBits = (short) (sig & 0xF);
        if (isTiny && roundBits != 0) {
          JSoftFloat.float_raise(FloatExceptionFlags.float_flag_underflow.getValue());
        }
      } else if ((0x1D < exp) || (0x8000 <= (sig + roundIncrement))) {
        JSoftFloat.float_raise(FloatExceptionFlags.float_flag_overflow.getValue()
            | FloatExceptionFlags.float_flag_inexact.getValue());

        Float16 float16 = packToF16UI(sign, (short) 0x1F, (short) 0);
        float16.float16b = float16.float16b - (roundIncrement == 0 ? 1 : 0);
        return float16;
      }
    }

    if (roundBits != 0) {
      JSoftFloat.float_exception_flags |= FloatExceptionFlags.float_flag_inexact.getValue();
    }

    sig = (short) ((sig + roundIncrement) >>> 4);
    sig &= ~((((roundBits ^ 8) == 0) & roundNearestEven) ? 1 : 0);

    return packToF16UI(sign, sig != 0 ? (short) exp : (short) 0, sig);
  }

  public static Float16 float16_add(Float16 a, Float16 b) {
    return new Float16((short)0);
    // TODO add16
  }

  public static Float16 float16_sub(Float16 a, Float16 b) {
    return new Float16((short)0);
    // TODO sub16
  }

  public static Float16 float16_mul(Float16 a, Float16 b) {
    return new Float16((short)0);
    // TODO mul16
  }

  public static Float16 float16_div(Float16 a, Float16 b) {
    return new Float16((short)0);
    // TODO div16
  }
}
